#!/bin/sh
# Copyright (c) 2000-2015 Synology Inc. All rights reserved.

PACKAGE_NAME="SafeAccess"
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
DNSFILTER_HTTPD_SH="${PACKAGE_TARGET_DIR}/scripts/dnsfilter-httpd-sys.sh"
DNSFILTER_IPTABLES_SH="${PACKAGE_TARGET_DIR}/scripts/dnsfilter-iptables.sh"
BIN_SYNO_SERVICE="/usr/syno/sbin/synoservicecfg"

#
# Restart the apache server to start/stop listening on the
# redirection ports in order to display the "DNS is blocked" page.
#
# IMPORTANT NOTE:
# 1. This function must be used with "&" (fork and execute) to
#    prevent restart the apache server before returning from a webapi.
#    (which casues operation failed)
#
# @see httpd_sys_check_restart
#
httpd_sys_restart ()
{
	local szf_script_lock="/tmp/dnsfilter_httpd_restart.lock"

	(
		if flock -n -x 8; then
			sleep 20
			${DNSFILTER_HTTPD_SH} install_httpd
			${DNSFILTER_IPTABLES_SH} restart
			flock -u 8
		fi
	)8> ${szf_script_lock}
}

#
# Restart the URL blocker if the IP address of either lbr* or
# gbr* interface is changed.
#
# Since the URL blocker (or DNS filter) will redirect the blocked
# HTTP/HTTPS requests to the IP address of the lbr0/gbr0 interface,
# the URL blocker must be aware of address changes.
#
# @param $1 [IN]
#		The NIC name that changed the IP address.
#
url_blocker_post()
{
	local iface=$1

	if [ $# -lt 1 ]; then
		echo "${FUNCNAME} failed - bad parameters"
		exit 1
	fi

	if [ ${iface} != "${iface/lbr}" ] || [ ${iface} != "${iface/gbr}" ]; then
		httpd_sys_restart &
	fi
}

case $1 in
	--sdk-mod-ver)
		# Print SDK support version
		echo "1.0";
	;;
	--name)
		# Print package name
		echo "url-blocker"
	;;
	--pkg-ver)
		# Print package version
		echo "1.0";
	;;
	--vendor)
		# Print package vendor
		echo "Synology";
	;;
	--pre)
	;;
	--post)
		url_blocker_post ${IFNAME}
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac
