#!/bin/sh
# Copyright (c) 2000-2018 Synology Inc. All rights reserved.

PACKAGE_NAME=${PACKAGE_NAME:-"SafeAccess"}
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
PACKAGE_WEBAPI_DIR="${PACKAGE_TARGET_DIR}/webapi"
PACKAGE_BIN_DIR="${PACKAGE_TARGET_DIR}/bin"
PACKAGE_CONF_DIR="/usr/syno/etc/packages/${PACKAGE_NAME}"

UPSTART_DIR="/etc/init"
DATABASE_UPDATE_HOOK_DIR="/usr/libexec/database_autoupdate/version_change"
TOPOLOGY_HOOK_DIR="/usr/libexec/net/topology_change/"

PAUSE_CHAIN="ACCESS_BAN_FILTER_FORWARD"
BLOCK_CHAIN="ACCESS_FILTER_FORWARD"
BLOCKPAGE_CHAIN="ACCESS_BLOCK_PREROUTING"

PAUSE_SET_V4="AC_BAN_MACS"
PAUSE_SET_V6="AC_BAN_MACS"
ACCESS_PASS_MACS_V4="access_pass_macs"
ACCESS_PASS_MACS_V6="access_pass_macs"
BLOCKTIME_BLOCK_MACS_V4="blocktime_block_macs"
BLOCKTIME_BLOCK_MACS_V6="blocktime_block_macs"
TIMEQUOTA_BLOCK_MACS_V4="timequota_block_macs"
TIMEQUOTA_BLOCK_MACS_V6="timequota_block_macs"
BLOCKTIME_BLOCK_IFACES_V4="blocktime_block_ifaces_v4"
BLOCKTIME_BLOCK_IFACES_V6="blocktime_block_ifaces_v6"
TIMEQUOTA_BLOCK_IFACES_V4="timequota_block_ifaces_v4"
TIMEQUOTA_BLOCK_IFACES_V6="timequota_block_ifaces_v6"

BLOCKTIME_BLOCKPAGE_HTTP_PORT="5032"
BLOCKTIME_BLOCKPAGE_HTTPS_PORT="5033"
TIMEQUOTA_BLOCKPAGE_HTTP_PORT="5042"
TIMEQUOTA_BLOCKPAGE_HTTPS_PORT="5043"

ipt() {
	local version=$1
	[ "-4" = $version ] && echo "iptables" && return 0
	[ "-6" = $version ] && echo "ip6tables" && return 0

	error_log "Iptables version: '$version' is not supported"
	return 1
}

error_log() {
	echo $1
	/usr/bin/logger -p err $1
}

ipset_exists() {
	local name=$1
	[ `ipset list $name 2>&1 | grep "not exist" | wc -l` -eq 0 ] && return 0
	return 1
}

create_matchset() {
	ipset_exists ${BLOCKTIME_BLOCK_IFACES_V4} || ipset create ${BLOCKTIME_BLOCK_IFACES_V4} hash:net,iface family inet
	ipset_exists ${TIMEQUOTA_BLOCK_IFACES_V4} || ipset create ${TIMEQUOTA_BLOCK_IFACES_V4} hash:net,iface family inet
	ipset_exists ${BLOCKTIME_BLOCK_IFACES_V6} || ipset create ${BLOCKTIME_BLOCK_IFACES_V6} hash:net,iface family inet6
	ipset_exists ${TIMEQUOTA_BLOCK_IFACES_V6} || ipset create ${TIMEQUOTA_BLOCK_IFACES_V6} hash:net,iface family inet6

	ipset_exists ${PAUSE_SET_V4} || ipset create ${PAUSE_SET_V4} hash:mac
	ipset_exists ${ACCESS_PASS_MACS_V4} || ipset create ${ACCESS_PASS_MACS_V4} hash:mac
	ipset_exists ${BLOCKTIME_BLOCK_MACS_V4} || ipset create ${BLOCKTIME_BLOCK_MACS_V4} hash:mac
	ipset_exists ${TIMEQUOTA_BLOCK_MACS_V4} || ipset create ${TIMEQUOTA_BLOCK_MACS_V4} hash:mac

	ipset_exists ${PAUSE_SET_V6} || ipset create ${PAUSE_SET_V6} hash:mac
	ipset_exists ${ACCESS_PASS_MACS_V6} || ipset create ${ACCESS_PASS_MACS_V6} hash:mac
	ipset_exists ${BLOCKTIME_BLOCK_MACS_V6} || ipset create ${BLOCKTIME_BLOCK_MACS_V6} hash:mac
	ipset_exists ${TIMEQUOTA_BLOCK_MACS_V6} || ipset create ${TIMEQUOTA_BLOCK_MACS_V6} hash:mac
}

destroy_matchset() {
	ipset_exists ${BLOCKTIME_BLOCK_IFACES_V4} && ipset destroy ${BLOCKTIME_BLOCK_IFACES_V4}
	ipset_exists ${TIMEQUOTA_BLOCK_IFACES_V4} && ipset destroy ${TIMEQUOTA_BLOCK_IFACES_V4}
	ipset_exists ${BLOCKTIME_BLOCK_IFACES_V6} && ipset destroy ${BLOCKTIME_BLOCK_IFACES_V6}
	ipset_exists ${TIMEQUOTA_BLOCK_IFACES_V6} && ipset destroy ${TIMEQUOTA_BLOCK_IFACES_V6}

	ipset_exists ${PAUSE_SET_V4} && ipset destroy ${PAUSE_SET_V4}
	ipset_exists ${ACCESS_PASS_MACS_V4} && ipset destroy ${ACCESS_PASS_MACS_V4}
	ipset_exists ${BLOCKTIME_BLOCK_MACS_V4} && ipset destroy ${BLOCKTIME_BLOCK_MACS_V4}
	ipset_exists ${TIMEQUOTA_BLOCK_MACS_V4} && ipset destroy ${TIMEQUOTA_BLOCK_MACS_V4}

	ipset_exists ${PAUSE_SET_V6} && ipset destroy ${PAUSE_SET_V6}
	ipset_exists ${ACCESS_PASS_MACS_V6} && ipset destroy ${ACCESS_PASS_MACS_V6}
	ipset_exists ${BLOCKTIME_BLOCK_MACS_V6} && ipset destroy ${BLOCKTIME_BLOCK_MACS_V6}
	ipset_exists ${TIMEQUOTA_BLOCK_MACS_V6} && ipset destroy ${TIMEQUOTA_BLOCK_MACS_V6}
}


check_iptables_action() {
	local action=$1
	[ "-I" = $action -o "-A" = $action -o "-D" = $action ] && return 0

	error_log "Iptables action: '$action' is not supported"
	return 1
}

set_lan_to_lan() {
	local iptver=$1
	local action=$2
	local chain=$3

	check_iptables_action $action || return 1

	`ipt $iptver` -t filter $action $chain -i lbr0 -o gbr0 -j RETURN
	`ipt $iptver` -t filter $action $chain -i gbr0 -o lbr0 -j RETURN
	`ipt $iptver` -t filter $action $chain -i gbr0 -o gbr0 -j RETURN
	`ipt $iptver` -t filter $action $chain -i lbr0 -o lbr0 -j RETURN
}

set_blocking_rule() {
	local iptver=$1
	local action=$2
	local chain=$3
	local matchset=$4
	local jump=$5

	check_iptables_action $action || return 1

	`ipt $iptver` -t filter $action $chain -m set --match-set $matchset src -j $jump
	`ipt $iptver` -t filter $action $chain -m set --match-set $matchset dst -j $jump
}

set_blockpage_redirect() {
	local iptver=$1
	local action=$2
	local matchset=$3
	local fromport=$4
	local toport=$5

	check_iptables_action $action || return 1

	`ipt $iptver` -t nat $action ${BLOCKPAGE_CHAIN} -m set --match-set $matchset src -p tcp --dport $fromport -j REDIRECT --to-ports $toport
	`ipt $iptver` -t nat $action ${BLOCKPAGE_CHAIN} -m set --match-set $matchset dst -p tcp --dport $fromport -j REDIRECT --to-ports $toport
}

set_blockpage_rule() {
	local iptver=$1
	local action=$2
	local matchset=$3
	local jump=$4

	check_iptables_action $action || return 1

	`ipt $iptver` -t nat $action ${BLOCKPAGE_CHAIN} -m set --match-set $matchset src -j $jump
	`ipt $iptver` -t nat $action ${BLOCKPAGE_CHAIN} -m set --match-set $matchset dst -j $jump
}

install_iptables() {
	# v4
	set_lan_to_lan -4 -A ${PAUSE_CHAIN}
	set_blocking_rule -4 -A ${PAUSE_CHAIN} ${PAUSE_SET_V4} DROP

	set_lan_to_lan -4 -A ${BLOCK_CHAIN}
	set_blocking_rule -4 -A ${BLOCK_CHAIN} ${ACCESS_PASS_MACS_V4} RETURN
	set_blocking_rule -4 -A ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_MACS_V4} DROP
	set_blocking_rule -4 -A ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_MACS_V4} DROP
	set_blocking_rule -4 -A ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_IFACES_V4} DROP
	set_blocking_rule -4 -A ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_IFACES_V4} DROP

	`ipt -4` -t nat -I ${BLOCKPAGE_CHAIN} -m state --state RELATED,ESTABLISHED -j RETURN
	set_blockpage_redirect -4 -A ${PAUSE_SET_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -A ${PAUSE_SET_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}

	set_blockpage_rule -4 -A ${ACCESS_PASS_MACS_V4} RETURN
	set_blockpage_redirect -4 -A ${BLOCKTIME_BLOCK_MACS_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -A ${BLOCKTIME_BLOCK_MACS_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -A ${TIMEQUOTA_BLOCK_MACS_V4} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -A ${TIMEQUOTA_BLOCK_MACS_V4} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -A ${BLOCKTIME_BLOCK_IFACES_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -A ${BLOCKTIME_BLOCK_IFACES_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -A ${TIMEQUOTA_BLOCK_IFACES_V4} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -A ${TIMEQUOTA_BLOCK_IFACES_V4} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}


	# v6
	set_lan_to_lan -6 -A ${PAUSE_CHAIN}
	set_blocking_rule -6 -A ${PAUSE_CHAIN} ${PAUSE_SET_V6} DROP

	set_lan_to_lan -6 -A ${BLOCK_CHAIN}
	set_blocking_rule -6 -A ${BLOCK_CHAIN} ${ACCESS_PASS_MACS_V6} RETURN
	set_blocking_rule -6 -A ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_MACS_V6} DROP
	set_blocking_rule -6 -A ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_MACS_V6} DROP
	set_blocking_rule -6 -A ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_IFACES_V6} DROP
	set_blocking_rule -6 -A ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_IFACES_V6} DROP

	`ipt -6` -t nat -I ${BLOCKPAGE_CHAIN} -m state --state RELATED,ESTABLISHED -j RETURN
	set_blockpage_redirect -6 -A ${PAUSE_SET_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -A ${PAUSE_SET_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}

	set_blockpage_rule -6 -A ${ACCESS_PASS_MACS_V6} RETURN
	set_blockpage_redirect -6 -A ${BLOCKTIME_BLOCK_MACS_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -A ${BLOCKTIME_BLOCK_MACS_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -A ${TIMEQUOTA_BLOCK_MACS_V6} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -A ${TIMEQUOTA_BLOCK_MACS_V6} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -A ${BLOCKTIME_BLOCK_IFACES_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -A ${BLOCKTIME_BLOCK_IFACES_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -A ${TIMEQUOTA_BLOCK_IFACES_V6} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -A ${TIMEQUOTA_BLOCK_IFACES_V6} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
}

uninstall_iptables() {
	# v4
	set_lan_to_lan -4 -D ${PAUSE_CHAIN}
	set_blocking_rule -4 -D ${PAUSE_CHAIN} ${PAUSE_SET_V4} DROP

	set_lan_to_lan -4 -D ${BLOCK_CHAIN}
	set_blocking_rule -4 -D ${BLOCK_CHAIN} ${ACCESS_PASS_MACS_V4} RETURN
	set_blocking_rule -4 -D ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_MACS_V4} DROP
	set_blocking_rule -4 -D ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_MACS_V4} DROP
	set_blocking_rule -4 -D ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_IFACES_V4} DROP
	set_blocking_rule -4 -D ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_IFACES_V4} DROP

	`ipt -4` -t nat -D ${BLOCKPAGE_CHAIN} -m state --state RELATED,ESTABLISHED -j RETURN
	set_blockpage_redirect -4 -D ${PAUSE_SET_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -D ${PAUSE_SET_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}

	set_blockpage_rule -4 -D ${ACCESS_PASS_MACS_V4} RETURN
	set_blockpage_redirect -4 -D ${BLOCKTIME_BLOCK_MACS_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -D ${BLOCKTIME_BLOCK_MACS_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -D ${TIMEQUOTA_BLOCK_MACS_V4} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -D ${TIMEQUOTA_BLOCK_MACS_V4} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -D ${BLOCKTIME_BLOCK_IFACES_V4} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -D ${BLOCKTIME_BLOCK_IFACES_V4} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -4 -D ${TIMEQUOTA_BLOCK_IFACES_V4} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -4 -D ${TIMEQUOTA_BLOCK_IFACES_V4} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}

	# v6
	set_lan_to_lan -6 -D ${PAUSE_CHAIN}
	set_blocking_rule -6 -D ${PAUSE_CHAIN} ${PAUSE_SET_V6} DROP

	set_lan_to_lan -6 -D ${BLOCK_CHAIN}
	set_blocking_rule -6 -D ${BLOCK_CHAIN} ${ACCESS_PASS_MACS_V6} RETURN
	set_blocking_rule -6 -D ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_MACS_V6} DROP
	set_blocking_rule -6 -D ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_MACS_V6} DROP
	set_blocking_rule -6 -D ${BLOCK_CHAIN} ${BLOCKTIME_BLOCK_IFACES_V6} DROP
	set_blocking_rule -6 -D ${BLOCK_CHAIN} ${TIMEQUOTA_BLOCK_IFACES_V6} DROP

	`ipt -6` -t nat -D ${BLOCKPAGE_CHAIN} -m state --state RELATED,ESTABLISHED -j RETURN
	set_blockpage_redirect -6 -D ${PAUSE_SET_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -D ${PAUSE_SET_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}

	set_blockpage_rule -6 -D ${ACCESS_PASS_MACS_V6} RETURN
	set_blockpage_redirect -6 -D ${BLOCKTIME_BLOCK_MACS_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -D ${BLOCKTIME_BLOCK_MACS_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -D ${TIMEQUOTA_BLOCK_MACS_V6} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -D ${TIMEQUOTA_BLOCK_MACS_V6} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -D ${BLOCKTIME_BLOCK_IFACES_V6} 80 ${BLOCKTIME_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -D ${BLOCKTIME_BLOCK_IFACES_V6} 443 ${BLOCKTIME_BLOCKPAGE_HTTPS_PORT}
	set_blockpage_redirect -6 -D ${TIMEQUOTA_BLOCK_IFACES_V6} 80 ${TIMEQUOTA_BLOCKPAGE_HTTP_PORT}
	set_blockpage_redirect -6 -D ${TIMEQUOTA_BLOCK_IFACES_V6} 443 ${TIMEQUOTA_BLOCKPAGE_HTTPS_PORT}
}

install_project() {
	create_matchset
	# install_iptables will be called on dnsfilter-iptables when profile enabled

	mkdir -p ${DATABASE_UPDATE_HOOK_DIR}
	${PACKAGE_BIN_DIR}/synoactool --init-database &
	${PACKAGE_BIN_DIR}/synoac_general_trigger --enable
	${PACKAGE_BIN_DIR}/synoac_clean_db_trigger --enable
	ln -sf ${PACKAGE_BIN_DIR}/synoac_update_db_hook ${DATABASE_UPDATE_HOOK_DIR}/synoac_update_db_hook
	ln -sf ${PACKAGE_BIN_DIR}/topology_hook.sh ${TOPOLOGY_HOOK_DIR}/accesscontrol.sh
	cp ${PACKAGE_TARGET_DIR}/etc/synoaccesscontrol/synotimespentd.upstart ${UPSTART_DIR}/synotimespentd.conf
	cp ${PACKAGE_TARGET_DIR}/etc/synoaccesscontrol/synoac_reload_ban_status.upstart ${UPSTART_DIR}/synoac_reload_ban_status.conf
}

uninstall_project() {
	uninstall_iptables
	destroy_matchset

	${PACKAGE_BIN_DIR}/synoac_general_trigger --disable
	${PACKAGE_BIN_DIR}/synoac_clean_db_trigger --disable
	rm -f ${DATABASE_UPDATE_HOOK_DIR}/synoac_update_db_hook
	rm -f ${TOPOLOGY_HOOK_DIR}/accesscontrol.sh
	rm -f ${UPSTART_DIR}/synotimespentd.conf
	rm -f ${UPSTART_DIR}/synoac_reload_ban_status.conf
}

start_project()
{
	${PACKAGE_BIN_DIR}/synoac_general_trigger --refresh-ban-config-group
	start synotimespentd
}

stop_project()
{
	stop synotimespentd
}

case $1 in
	--install)
		install_project
		exit 0
	;;
	--uninstall)
		uninstall_project
		exit 0
	;;
	--install-iptables)
		install_iptables
		exit 0
	;;
	--uninstall-iptables)
		uninstall_iptables
		exit 0
	;;
	--start)
		start_project
		exit 0
	;;
	--stop)
		stop_project
		exit 0
	;;
	--status)
		exit 0
	;;
	--killall)
        ;;
	--log)
		exit 0
	;;
esac

exit 0
